#' Find marker genes using rankCorr
#'
#' @param sce A SingleCellExperiment object,
#' @param pars A list of parameters passed to the function.
#'
#' @return A list containing the time of the computation and results of the
#'  method.
#'
run_rankcorr <- function(sce, pars) {

  # Ensure we use the correct conda environment.
  # FIXME: Move to .Rprofile.
  Sys.setenv(RETICULATE_PYTHON = "/home/jpullin/.conda/envs/mage/bin/python")
  library(reticulate)

  old_wd <- getwd()

  # Needed for the import statement below to work
  rankcorr_path <- here::here("RankCorr")
  setwd(rankcorr_path)

  py_run_string("import picturedRocks")

  py$X <- t(as.matrix(counts(sce)))

  # rankCorr requires y to be 0-indexed.
  lookup <- (1:length(unique(colLabels(sce)))) - 1
  names(lookup) <- levels(colLabels(sce))
  # python requires real integers.
  y_vec <- as.integer(unname(lookup[colLabels(sce)]))
  # rankCorr needs a numpy array.
  y <- matrix(y_vec, ncol = 1)

  py$y <- y

  py_run_string("data = picturedRocks.Rocks(X, y)")
  # We write to file because that is the only way to get per-cluster markers..
  times <- system.time({
    py_run_string(
      glue::glue("markers = data.CSrankMarkers(lamb={pars$lambda}, writeOut=True)")
    )
  })

  file_name <- glue::glue("ovrRankGenes-lamb{pars$lambda}.dat")
  file_path <- file.path(rankcorr_path, file_name)

  raw_markers <- readLines(file_path)
  unlink(file_path)

  # The indexes are 0 indexed.
  mgs_inds <- lapply(
    raw_markers,
    function(x) as.numeric(unlist(strsplit(x, " "))) + 1
  )

  mgs <- lapply(
    mgs_inds,
    function(x) rownames(sce)[x]
  )

  result <- tibble::tibble(
    cluster = rep(names(lookup), lengths(mgs)),
    gene = unlist(mgs),
    log_fc = 0,
    raw_statistic = 0,
    scaled_statistic = 0,
    p_value = 0,
    p_value_adj = 0
  )

  # We use the 'elapsed' time.
  time <- times[[3]]

  # Restore the old working directory.
  setwd(old_wd)

  list(time = time, result = result, raw_result = raw_markers, pars = pars)
}
